/***************************************************************************
                          QApplication.m  -  description
                             -------------------
    begin                : Fri Mar 3 2000
    copyright            : (C) 2000 by Richard Dale
    email                : Richard_Dale@tipitina.demon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <Foundation/NSAutoreleasePool.h>
#include <qtobjc/QApplication.h>
#include <qtc/qtc.h>

static QApplication *			qAppInstance = nil;
static NSAutoreleasePool *	pool = nil;

@implementation QApplication

- initWithArgc: (int) argc argv: (char **) argv
{
	pool = [[NSAutoreleasePool alloc] init];
	[self setQt: qt_new_QApplication(argc, argv)];
	[super init];
	qAppInstance = self;
	return self;
}

#if defined(_WS_X11_)
- initWithDisplay: (Display *) dpy
{
	[super init];
	[self setQt: qt_new_QApplication1([self qt] , dpy)];
	qAppInstance = self;
	return self;
}
#endif

- (void) dealloc
{
	qt_del_QApplication([self qt]);
	[super dealloc];
	return;
}

+ qApp
{
	return qAppInstance;
}

- (int) argc
{
	return (int) qt_QApplication_argc([self qt]);
}

- (char **) argv
{
	return (char **) qt_QApplication_argv([self qt]);
}

+ (GUIStyle) style
{
	return (GUIStyle) qt_QApplication_style();
}

+ setStyle: (GUIStyle) style
{
	qt_QApplication_setStyle(style);
	return self;
}

+ (int) colorSpec
{
	return (int) qt_QApplication_colorSpec();
}

+ setColorSpec: (int) spec
{
	qt_QApplication_setColorSpec(spec);
	return self;
}

+ (QCursor *) overrideCursor
{
	return (QCursor *) [QObject objectForQtKey: (void *) qt_QApplication_overrideCursor()];
}

+ setOverrideCursor: (QCursor *) cursor replace: (BOOL) yn
{
	qt_QApplication_setOverrideCursor((qt_QCursor *) [cursor qt], yn);
	return self;
}

+ restoreOverrideCursor
{
	qt_QApplication_restoreOverrideCursor();
	return self;
}

+ (BOOL) hasGlobalMouseTracking
{
	return (BOOL) qt_QApplication_hasGlobalMouseTracking();
}

+ setGlobalMouseTracking: (BOOL) enable
{
	qt_QApplication_setGlobalMouseTracking(enable);
	return self;
}

+ (QPalette *) palette
{
	return (QPalette *) [QObject objectForQtKey: (void *) qt_QApplication_palette()];
}

+ setPalette: (QPalette *) palette updateAllWidgets: (BOOL) yn
{
	qt_QApplication_setPalette((qt_QPalette *) [palette qt], yn);
	return self;
}

+ font
{
	return (QFont *) qt_QApplication_font();
}

+ setFont: (QFont *) font
{
	qt_QApplication_setFont((qt_QFont *) [font qt], NO);
	return self;
}

+ setFont: (QFont *) font updateAllWidgets: (BOOL) yn
{
	qt_QApplication_setFont((qt_QFont *) [font qt], yn);
	return self;
}

+ (QFontMetrics *) fontMetrics
{
	return (QFontMetrics *) [QObject objectForQtKey: (void *) qt_QApplication_fontMetrics()];
}


- (QWidget *) mainWidget
{
	return (QWidget *) [QObject objectForQtKey: (void *) qt_QApplication_mainWidget([self qt])];
}

- setMainWidget: (QWidget *) widget
{
	qt_QApplication_setMainWidget(	[self qt],
									(qt_QWidget *) [widget qt] );
	return self;
}


+ (NSArray *) allWidgets
{
//	return (NSArray *) qt_QApplication_allWidgets();
	return nil;
}

+ (NSArray *) topLevelWidgets
{
//	return (NSArray *) qt_QApplication_topLevelWidgets();
	return nil;
}

+ (QWidget *) desktop
{
	return (QWidget *) [QObject objectForQtKey: (void *) qt_QApplication_desktop()];
}

+ (QWidget *) activePopupWidget
{
	return (QWidget *) [QObject objectForQtKey: (void *) qt_QApplication_activePopupWidget()];
}

+ (QWidget *) activeModalWidget
{
	return (QWidget *) [QObject objectForQtKey: (void *) qt_QApplication_activeModalWidget()];
}

+ (QClipboard *) clipboard
{
  	return (QClipboard *) [QObject objectForQtKey: (void *) qt_QApplication_clipboard()];
}

- (QWidget *) focusWidget
{
	return (QWidget *) [QObject objectForQtKey: (void *) qt_QApplication_focusWidget([self qt])];
}

+ (QWidget *) widgetAt: (int) x : (int) y child: (BOOL) yn
{
	return (QWidget *) [QObject objectForQtKey: (void *) qt_QApplication_widgetAt(x, y, yn)];
}

+ (QWidget *) widgetAt: (QPoint *) point child: (BOOL) yn
{
	return (QWidget *) [QObject objectForQtKey: (void *) qt_QApplication_widgetAt1((qt_QPoint *) [point qt], yn)];
}


- exec
{
	qt_QApplication_exec([self qt]);
	return self;
}

- processEvents
{
	qt_QApplication_processEvents([self qt]);
	return self;
}

- processEvents: (int) maxtime
{
	qt_QApplication_processEvents1([self qt], maxtime);
	return self;
}

- processOneEvent
{
	qt_QApplication_processOneEvent([self qt]);
	return self;
}

- (int) enterLoop
{
	return (int) qt_QApplication_enter_loop([self qt]);
}

- exitLoop
{
	qt_QApplication_exit_loop([self qt]);
	return self;
}

+ exit: (int) retcode
{
	qt_QApplication_exit(retcode);
	return self;
}


+ (BOOL)	sendEvent: (QObject *) receiver event: (QEvent *) event
{
	return (BOOL) qt_QApplication_sendEvent(	(qt_QObject *) [receiver qt],
											(qt_QEvent *) [(<QtSupport>) event qt] );
}

+ postEvent: (QObject *) receiver event: (QEvent *) event
{
	qt_QApplication_postEvent(	(qt_QObject *) [receiver qt],
								(qt_QEvent *) [(<QtSupport>) event qt] );
	return self;
}

+ sendPostedEvents: (QObject *) receiver eventType: (int) eventType
{
//	qt_QApplication_sendPostedEvents((qt_QObject *) receiver, eventType);
	return self;
}


- (BOOL) notify: (QObject *) receiver event: (QEvent *) event
{
	return (BOOL) qt_QApplication_notify(	[self qt],
										(qt_QObject *) [receiver qt],
										(qt_QEvent *) [(<QtSupport>) event qt] );
}


+ (BOOL)	startingUp
{
	return (BOOL) qt_QApplication_startingUp();
}

+ (BOOL)	closingDown
{
	return (BOOL) qt_QApplication_closingDown();
}


+ flushX
{
	qt_QApplication_flushX();
	return self;
}

+ syncX
{
	qt_QApplication_syncX();
	return self;
}

+ beep
{
	qt_QApplication_beep();
	return self;
}

+ setWinStyleHighlightColor: (QColor *) color
{
	qt_QApplication_setWinStyleHighlightColor((qt_QColor *) [color qt]);
	return self;
}

+ (QColor *) winStyleHighlightColor
{
	return (QColor *) [QObject objectForQtKey: (void *) qt_QApplication_winStyleHighlightColor()];
}


+ setDoubleClickInterval: (int) interval
{
	qt_QApplication_setDoubleClickInterval(interval);
	return self;
}

+ (int) doubleClickInterval
{
	return (int) qt_QApplication_doubleClickInterval();
}


#if defined(_WS_WIN_)
+ (WindowsVersion) winVersion
{
	return (WindowsVersion) qt_QApplication_winVersion([self qt]);
}

#endif

#if defined(_WS_X11_)
- (BOOL) x11EventFilter: (XEvent *) event
{
	return (BOOL) qt_QApplication_x11EventFilter(	[self qt],
												[event qt] );
}

- (int) x11ProcessEvent: (XEvent *) event
{
	return (int) qt_QApplication_x11ProcessEvent(	[self qt],
												[event qt] );
}

#endif


/*
 * Slots
 */
- quit
{
	qt_QApplication_quit([self qt]);
	return self;
}

@end

