/***************************************************************************
                          QListBox.m  -  description
                             -------------------
    begin                : Sat Apr 1 2000
    copyright            : (C) 2000 by Richard Dale
    email                : Richard_Dale@tipitina.demon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtobjc/QListBox.h>
#include <qtobjc/QPixmap.h>
#include <qtobjc/QPainter.h>

#include <qtc/qtc.h>

@implementation QListBoxItem
- (void *) qt
{
	return _qt;
}

- setQt: (void *) qt
{
	_qt = qt;
	return self;
}


- (NSString *) text
{
	return (NSString *) [NSString stringWithCString: qt_QListBoxItem_text([self qt])];
}


- (QPixmap *) pixmap
{
	return (QPixmap *) [QObject objectForQtKey: qt_QListBoxItem_pixmap([self qt]) withClass: [QPixmap class]];
}


- (int) height: (QListBox *) listBox
{
	return (int) qt_QListBoxItem_height([self qt], [listBox qt]);
}


- (int) width: (QListBox *) listBox
{
	return (int) qt_QListBoxItem_width([self qt], [listBox qt]);
}

@end

@implementation QListBoxText

- initWithText: (NSString *) text
{
	[self setQt: qt_new_QListBoxText((char *) [text cString])];
	[super init];	
	return self;
}


- (void) dealloc
{
	qt_del_QListBoxText([self qt]);	
	[super dealloc];	
	return;
}


- paint: (QPainter *) painter
{
	qt_QListBoxText_paint([self qt], [painter qt]);
	return self;
}


- (int) height: (QListBox *) listBox
{
	return (int) qt_QListBoxText_height([self qt], [listBox qt]);
}


- (int) width: (QListBox *) listBox
{
	return (int) qt_QListBoxText_width([self qt], [listBox qt]);
}

@end

@implementation QListBoxPixmap

- initWithPixmap: (QPixmap *) pixmap
{
	[self setQt: qt_new_QListBoxPixmap([pixmap qt])];
	[super init];
	return self;
}

- (QPixmap *) pixmap
{
	return (QPixmap *) [QObject objectForQtKey: qt_QListBoxPixmap_pixmap([self qt])];
}

@end

@implementation QListBox

- initWithParent: (QWidget *) parent name: (NSString *) name flags: (WFlags) f
{
	[self setQt: qt_new_QListBox([parent qt], (char *) [name cString], f)];	
	return self;
}


- (void) dealloc
{
	qt_del_QListBox([self qt]);	
	[super dealloc];	
	return;
}


- setFont: (QFont *) font
{
	qt_QListBox_setFont([self qt], [font qt]);
	return self;
}


- (uint) count
{
	return (uint) qt_QListBox_count([self qt]);
}


- insertListBoxItem: (QListBoxItem *) item
{
	return [self insertListBoxItem: item index: -1];
}


- insertListBoxItem: (QListBoxItem *) item index: (int) index
{
	qt_QListBox_insertItem([self qt], [item qt], index);
	return self;
}

- insertTextItem:  (NSString *) text
{
	return [self insertTextItem: text index: -1];
}


- insertTextItem: (NSString *) text index: (int) index
{
	qt_QListBox_insertItem1([self qt], (char *) [text cString], index);
	return self;
}


- insertPixmapItem: (QPixmap *) pixmap
{
	return [self insertPixmapItem: pixmap index: -1];
}

- insertPixmapItem: (QPixmap *) pixmap index: (int) index
{
	qt_QListBox_insertItem2([self qt], [pixmap qt], index);
	return self;
}


- inListBoxSort: (QListBoxItem *) item
{
	qt_QListBox_inSort([self qt], [item qt]);
	return self;
}


- inTextSort: (NSString *) text
{
	qt_QListBox_inSort1([self qt], (char *) [text cString]);
	return self;
}


- removeItem: (int) index
{
	qt_QListBox_removeItem([self qt], index);
	return self;
}


- clear
{
	qt_QListBox_clear([self qt]);
	return self;
}


- (NSString *) text: (int) index
{
	return (NSString *) [NSString stringWithCString: qt_QListBox_text([self qt], index)];
}


- (QPixmap *) pixmap: (int) index
{
	return (QPixmap *) [QObject objectForQtKey: qt_QListBox_pixmap([self qt], index) withClass: [QPixmap class]];
}


- changeListBoxItem: (QListBoxItem *) item index: (int) index
{
	qt_QListBox_changeItem([self qt], [item qt], index);
	return self;
}


- changeStringItem: (NSString *) text index: (int) index
{
	qt_QListBox_changeItem1([self qt], (char *) [text cString], index);
	return self;
}


- changePixmapItem: (QPixmap *) pixmap index: (int) index
{
	qt_QListBox_changeItem2([self qt], [pixmap qt], index);
	return self;
}


- (BOOL) autoUpdate
{
	return (BOOL) qt_QListBox_autoUpdate([self qt]);
}


- setAutoUpdate: (BOOL) yn
{
	qt_QListBox_setAutoUpdate([self qt], yn);
	return self;
}


- (int) numItemsVisible
{
	return (int) qt_QListBox_numItemsVisible([self qt]);
}


- setFixedVisibleLines: (int) lines
{
//	qt_QListBox_setFixedVisibleLines([self qt], lines);
	return self;
}


- (int) currentItem
{
	return (int) qt_QListBox_currentItem([self qt]);
}


- setCurrentItem: (int) index
{
	qt_QListBox_setCurrentItem([self qt], index);
	return self;
}


- centerCurrentItem
{
	qt_QListBox_centerCurrentItem([self qt]);
	return self;
}


- (int) topItem
{
	return (int) qt_QListBox_topItem([self qt]);
}


- setTopItem: (int) index
{
	qt_QListBox_setTopItem([self qt], index);
	return self;
}


- setBottomItem: (int) index
{
//	qt_QListBox_setBottomItem([self qt], index);
	return self;
}


- (BOOL) dragSelect
{
	return (BOOL) qt_QListBox_dragSelect([self qt]);
}


- setDragSelect: (BOOL) yn
{
	qt_QListBox_setDragSelect([self qt], yn);
	return self;
}


- (BOOL) autoScroll
{
	return (BOOL) qt_QListBox_autoScroll([self qt]);
}


- setAutoScroll: (BOOL) yn
{
	qt_QListBox_setAutoScroll([self qt], yn);
	return self;
}


- (BOOL) autoScrollBar
{
	return (BOOL) qt_QListBox_autoScrollBar([self qt]);
}


- setAutoScrollBar: (BOOL) yn
{
	qt_QListBox_setAutoScrollBar([self qt], yn);
	return self;
}


- (BOOL) scrollBar
{
	return (BOOL) qt_QListBox_scrollBar([self qt]);
}


- setScrollBar: (BOOL) yn
{
	qt_QListBox_setScrollBar([self qt], yn);
	return self;
}


- (BOOL) autoBottomScrollBar
{
	return (BOOL) qt_QListBox_autoBottomScrollBar([self qt]);
}


- setAutoBottomScrollBar: (BOOL) yn
{
	qt_QListBox_setAutoBottomScrollBar([self qt], yn);
	return self;
}


- (BOOL) bottomScrollBar
{
	return (BOOL) qt_QListBox_bottomScrollBar([self qt]);
}


- setBottomScrollBar: (BOOL) yn
{
	qt_QListBox_setBottomScrollBar([self qt], yn);
	return self;
}


- (BOOL) smoothScrolling
{
	return (BOOL) qt_QListBox_smoothScrolling([self qt]);
}


- setSmoothScrolling: (BOOL) yn
{
	qt_QListBox_setSmoothScrolling([self qt], yn);
	return self;
}


- (int) itemHeight
{
	return (int) qt_QListBox_itemHeight([self qt]);
}


- (int) itemHeight: (int) index
{
	return (int) qt_QListBox_itemHeight1([self qt], index);
}


- (long) maxItemWidth
{
	return (long) qt_QListBox_maxItemWidth([self qt]);
}


- (BOOL) isMultiSelection
{
	return (BOOL) qt_QListBox_isMultiSelection([self qt]);
}


- setMultiSelection: (BOOL) yn
{
	qt_QListBox_setMultiSelection([self qt], yn);
	return self;
}


- setSelected: (int) index selected: (BOOL) yn
{
	qt_QListBox_setSelected([self qt], index, yn);
	return self;
}


- (BOOL) isSelected: (int) index
{
	return (BOOL) qt_QListBox_isSelected([self qt], index);
}


- (QSize *) sizeHint
{
//	return (QSize *) [QObject objectForQtKey: qt_QListBox_sizeHint([self qt]) withClass: [QSize class]];
	return nil;
}

@end

