/***************************************************************************
                          QObject.m  -  description
                             -------------------
    begin                : Wed Mar 8 2000
    copyright            : (C) 2000 by Richard Dale
    email                : Richard_Dale@tipitina.demon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <Foundation/NSValue.h>
#include <Foundation/NSDictionary.h>
#include <Foundation/NSInvocation.h>
#include <Foundation/NSMethodSignature.h>

#include <qtobjc/QObject.h>
#include <qtc/qtc.h>

static NSMutableDictionary *		qtKeyToObjcMap = nil;
static NSMutableDictionary *		qtSignalDictionary = nil;

void
ObjcSlotCallback(void * userData, void * value)
{
	NSInvocation *	invocation;
	invocation = (NSInvocation *) userData;
	
	if ([[invocation methodSignature] numberOfArguments] > 2) {
		[invocation setArgument: value atIndex: 2];
	}
	
	[invocation invoke];
}

@implementation  QObject

- (void) dealloc
{
	qt_del_QObject([self qt]);
	[super dealloc];
	return;
}

+ (BOOL) connect: (<QObjectInterface,QtSupport>) sender signal: (NSString *) signal receiver: (<QObjectInterface,QtSupport>) receiver slot: (NSString *) member
{
	qt_Slot *			slotId;
	
	if ([member hasSuffix: @")"]) {
	 	if (	!qt_QObject_connect(	([signal hasSuffix: @")"]) ? [sender qt] : [QObject signalIdForSender: sender signal: signal],
 								(char *)[[NSString stringWithFormat: @"2%@", signal] cString],
 								[receiver qt],
 								(char *)[[NSString stringWithFormat: @"1%@", member] cString] ) )
 		{
 			NSLog(@"connect failed...signal: %@ slot: %@", signal, member);
 			return NO;
 		}
	} else {
		slotId = (qt_Slot *) [QObject slotIdForReceiver: receiver slot: member];
		
		if ([signal hasSuffix: @")"]) {
			if (		!qt_QObject_connect(	[sender qt],
										(char *)[[NSString stringWithFormat: @"2%@", signal] cString],
										(qt_QObject *) slotId,
										([member hasSuffix: @":"]) ?
											([signal hasSuffix: @"(int)"] ? "1UserDataSlot(int)" : "1UserDataSlot(void *)")
											: "1UserDataSlot()" ) )
			{
 				NSLog(@"connect failed...signal: %@ slot: %@", signal, member);
 				return NO;
			}
		} else {
			if (		!qt_QObject_connect(	[QObject signalIdForSender: sender signal: signal],
										([signal hasSuffix: @":"]) ? "2pointerSignal(void *)" : "2voidSignal()",
										(qt_QObject *) slotId,
										([member hasSuffix: @":"]) ? "1UserDataSlot(void *)" : "1UserDataSlot()" ) )
			{
 				NSLog(@"connect failed...signal: %@ slot: %@", signal, member);
 				return NO;
			}
		}
	}
	
	return YES;
}

+ (BOOL) disconnect: (<QObjectInterface,QtSupport>) sender signal: (NSString *) signal receiver: (<QObjectInterface,QtSupport>) receiver slot: (NSString *) member
{
//	return (BOOL) qt_QObject_disconnect([self qt]);
	return NO;
}

+ (void *) signalIdForSender: (<QObjectInterface,QtSupport>) sender signal: (NSString *) signal
{
	NSValue *	qtSignal;
	
	qtSignal = [qtSignalDictionary objectForKey: [NSString stringWithFormat: @"%@%@", sender, signal]];
	
	if (qtSignal == nil) {
		qtSignal = [NSValue valueWithPointer: qt_new_Signal()];
		[qtSignalDictionary setObject: qtSignal forKey: [NSString stringWithFormat: @"%@%@", sender, signal]];
	}
	
	return [qtSignal pointerValue];
}

+ sender: (<QObjectInterface,QtSupport>) sender emit: (NSString *) signal
{
	NSValue *	qtSignal;
	
	qtSignal = [qtSignalDictionary objectForKey: [NSString stringWithFormat: @"%@%@", sender, signal]];
	qt_emitVoid([qtSignal pointerValue]);
	
	return self;
}

+ sender: (<QObjectInterface,QtSupport>) sender emit: (NSString *) signal intValue: (int) value
{
	NSValue *	qtSignal;
	
	qtSignal = [qtSignalDictionary objectForKey: [NSString stringWithFormat: @"%@%@", sender, signal]];
	
	qt_emitPointer([qtSignal pointerValue], (void *) value);
	return self;
}

+ sender: (<QObjectInterface,QtSupport>) sender emit: (NSString *) signal value: value
{
	NSValue *	qtSignal;
	
	qtSignal = [qtSignalDictionary objectForKey: [NSString stringWithFormat: @"%@%@", sender, signal]];
	
	qt_emitPointer([qtSignal pointerValue], (void *) value);
	return self;
}

+ (int) slotIdForReceiver: (<QObjectInterface,QtSupport>) receiver slot: (NSString *) member
{
	SEL					selector;
	NSInvocation *		invocation;
	NSMethodSignature *	signature;
	qt_Slot *			slotId;
	
	selector = sel_get_uid([member cString]);
	signature = [[(<NSObject>) receiver class] instanceMethodSignatureForSelector: selector];
	invocation = [NSInvocation invocationWithMethodSignature: signature];
	[invocation setSelector: selector];
	[invocation setTarget: receiver];
	slotId = qt_new_UserDataSlot(ObjcSlotCallback, (void *) invocation);
	
	return (int) slotId;
}

@end

@implementation  QObject (QtSupport)

+ (void) initialize
{
	qtKeyToObjcMap = [[NSMutableDictionary alloc] initWithCapacity: 100];
	qtSignalDictionary = [[NSMutableDictionary alloc] initWithCapacity: 100];
	return;
}

+ setObject: anId forQtKey: (void *) qt
{
//	NSLog(@"setObject: %@ key: %8.8x", anId, qt);
	[qtKeyToObjcMap setObject: anId forKey: [NSString stringWithFormat: @"%8.8x", qt]];
	[anId release];
	return self;
}

+ removeQtKey: (void *) qt
{
//	NSLog(@"removeQtKey: %8.8x", qt);
	[qtKeyToObjcMap removeObjectForKey: [NSString stringWithFormat: @"%8.8x", qt]];
	return self;
}

+ objectForQtKey: (void *) qt
{
	id		objcInstance;
	Class	objcClass;

	if (qt == (void *) NULL) {
		return nil;
	}
	
	objcInstance = [qtKeyToObjcMap objectForKey: [NSString stringWithFormat: @"%8.8x", qt]];

	/*
	 * If there is no existing mapping for the current qt instance,
	 * then get the C++ class name, and allocate an Objective-C
	 * instance with the same class name. This code assumes that
	 * all qt instances, without existing mappings, have a className()
	 * member function.
	 */
	if (objcInstance == nil) {
		objcClass = objc_get_class(qt_QObject_className(qt));
		objcInstance = [[objcClass alloc] init];
		[objcInstance setQt: qt];
		[self setObject: objcInstance forQtKey: qt];
	}
	
	return objcInstance;
}

+ objectForQtKey: (void *) qt withClass: (Class) objcClass
{
	id		objcInstance;

	if (qt == (void *) NULL) {
		return nil;
	}
	
	objcInstance = [qtKeyToObjcMap objectForKey: [NSString stringWithFormat: @"%8.8x", qt]];

	/*
	 * If there is no existing mapping for the current qt instance,
	 * then allocate an Objective-C instance with the given class.
	 * member function.
	 */
	if (objcInstance == nil) {
		objcInstance = [[[objcClass alloc] init] autorelease];
		[objcInstance setQt: qt];
		[self setObject: objcInstance forQtKey: qt];
	}
	
	return objcInstance;
}

- (void *) qt
{
	return _qt;
}

- setQt: (void *) qt
{
	_qt = qt;
	return self;
}

@end					

@implementation QObject (QObjectImplementation)

- init
{
	[super init];
	[QObject setObject: self forQtKey: [self qt]];
	return self;
}
				
- initWithParent: (QObject *) parent name: (NSString *) name
{
	[	self setQt:
			qt_new_QObject((qt_QObject *) parent, (char *) [name cString]) ];
	[self init];
	return self;
}


- (BOOL) event: (QEvent *) event
{
	return (BOOL) qt_QObject_event(	[self qt],
									(qt_QEvent *) [(<QtSupport>) event qt] );
}

- (BOOL) eventFilter: (QObject *) filter event: (QEvent *) event
{
	return (BOOL) qt_QObject_eventFilter(	[self qt],
										(qt_QObject *) [filter qt],
										(qt_QEvent *) [(<QtSupport>) event qt] );
}

- (NSString *) tr: (NSString *) string
{
//	return (NSString *) qt_QObject_tr([self qt], [string cString]);
	return nil;
}

- (NSString *) className
{
	return (NSString *) [NSString stringWithCString: qt_QObject_className([self qt])];
}

- (BOOL) isA: (NSString *) name
{
	return (BOOL) qt_QObject_isA([self qt], (char *) [name cString]);
}

- (BOOL) inherits: (NSString *) name
{
	return (BOOL) qt_QObject_inherits([self qt], (char *) [name cString]);
}

- setName: (NSString *) name
{
	qt_QObject_setName([self qt], (char *) [name cString]);
	return self;
}

- blockSignals: (BOOL) yn
{
	qt_QObject_blockSignals([self qt], yn);
	return self;
}

- (int) startTimer: (int) interval
{
	return (int) qt_QObject_startTimer([self qt], interval);
}

- killTimer: (int) identifier
{
	qt_QObject_killTimer([self qt], identifier);
	return self;
}

- killTimers
{
	qt_QObject_killTimers([self qt]);
	return self;
}

- (NSArray *) queryList: (NSString *) inheritsClass name: (NSString *) objname regexMatch: (BOOL) regex recursiveSearch: (BOOL) recursive
{
//	return (NSArray *) qt_QObject_queryList([self qt]);
	return nil;
}

- insertChild: (QObject *) child
{
	qt_QObject_insertChild([self qt], (qt_QObject *) [child qt]);
	return self;
}

- removeChild: (QObject *) child
{
	qt_QObject_removeChild([self qt], (qt_QObject *) [child qt]);
	return self;
}

- installEventFilter: (QObject *) filter
{
	qt_QObject_installEventFilter([self qt], (qt_QObject *) [filter qt]);
	return self;
}

- removeEventFilter: (QObject *) filter
{
	qt_QObject_removeEventFilter([self qt], (qt_QObject *) [filter qt]);
	return self;
}

- (BOOL) connect: (<QObjectInterface,QtSupport>) sender signal: (NSString *) signal slot: (NSString *) member
{
	return [QObject connect: sender signal: signal receiver: self slot: member];
}

- (BOOL) disconnect: (NSString *) signal receiver: (<QObjectInterface,QtSupport>) receiver slot: (NSString *) member
{
//	return (BOOL) qt_QObject_disconnect1([self qt]);
	return NO;
}

- (BOOL) disconnect: (<QObjectInterface,QtSupport>) receiver slot: (NSString *) member
{
//	return (BOOL) qt_QObject_disconnect([self qt]);
	return NO;
}

- emit: (NSString *) signal
{
	[QObject sender: self emit: signal];
	return self;
}

- emit: (NSString *) signal value: value
{
	[QObject sender: self emit: signal value: value];
	return self;
}

- emit: (NSString *) signal intValue: (int) value
{
	[QObject sender: self emit: signal intValue: value];
	return self;
}


- dumpObjectTree
{
	qt_QObject_dumpObjectTree([self qt]);
	return self;
}

- dumpObjectInfo
{
	qt_QObject_dumpObjectInfo([self qt]);
	return self;
}


- connectNotify: (NSString *) signal
{
//	qt_QObject_connectNotify([self qt]);
	return self;
}

- disconnectNotify: (NSString *) signal
{
//	qt_QObject_disconnectNotify([self qt]);
	return self;
}

- (BOOL) checkConnectArgs: (NSString *) signal receiver: (QObject *) receiver slot: (NSString *) member
{
//	return (BOOL) qt_QObject_checkConnectArgs([self qt]);
	return NO;
}

+ badSuperClassWarning: (NSString *) className superclass: (NSString *) superclass
{
//	qt_QObject_badSuperClassWarning([self qt]);
	return self;
}

@end

