/***************************************************************************
                          QPopupMenu.m  -  description
                             -------------------
    begin                : Tue Mar 14 2000
    copyright            : (C) 2000 by Richard Dale
    email                : Richard_Dale@tipitina.demon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <Foundation/NSInvocation.h>
#include <Foundation/NSMethodSignature.h>

#include <qtobjc/QPopupMenu.h>
#include <qtobjc/QMenuData.h>
#include <qtobjc/QObject.h>

#include <qtc/qtc.h>


@implementation QPopupMenu

- init
{
	[self initWithParent: nil name: nil];
	return self;
}

- initWithParent: (QWidget *) parent name: (NSString *) name
{
	[self setQt: qt_new_QPopupMenu([parent qt], (char *) [name cString])];
	return self;
}


- (void) dealloc
{
	qt_del_QPopupMenu([self qt]);	
	[super dealloc];	
	return;
}



- popup: (QPoint *) pos index: (int) indexAtPoint
{
	qt_QPopupMenu_popup([self qt], [pos qt], indexAtPoint);
	return self;
}


- updateItem: (int) identifier
{
	qt_QPopupMenu_updateItem([self qt], identifier);
	return self;
}


- setCheckable: (BOOL) yn
{
	qt_QPopupMenu_setCheckable([self qt], yn);
	return self;
}


- (BOOL) isCheckable
{
	return (BOOL) qt_QPopupMenu_isCheckable([self qt]);
}


- setFont: (QFont *) font
{
	qt_QPopupMenu_setFont([self qt], [font qt]);
	return self;
}

	
- show
{
	qt_QPopupMenu_show([self qt]);
	return self;
}

				
- hide
{
	qt_QPopupMenu_hide([self qt]);
	return self;
}

				
- (int) exec
{
	return (int) qt_QPopupMenu_exec([self qt]);
}
				
- (int) exec: (QPoint *) point
{
	return (int) qt_QPopupMenu_exec1([self qt], [point qt]);
}


- setActiveItem: (int) item
{
	qt_QPopupMenu_setActiveItem([self qt], item);
	return self;
}

@end

@implementation QPopupMenu (QMenuDataImplementation)



- (uint) count
{
	return (uint) qt_QMenuData_count(qt_QPopupMenu_QMenuData([self qt]));
}


- (int) insertItem: (NSString *) text receiver: (<QObjectInterface>) receiver slot: (NSString *) member
{
	return [self insertItem: text receiver: receiver slot: member accel: 0 identifier: -1 index: -1];	
}

- (int) insertItem: (NSString *) text receiver: (<QObjectInterface>) receiver slot: (NSString *) member accel: (int) accel
{
	return [self insertItem: text receiver: receiver slot: member accel: accel identifier: -1 index: -1];	
}

- (int) insertItem: (NSString *) text receiver: (<QObjectInterface>) receiver slot: (NSString *) member accel: (int) accel identifier: (int) identifier
{
	return [self insertItem: text receiver: receiver slot: member accel: accel identifier: identifier index: -1];	
}

- (int) insertItem: (NSString *) text receiver: (<QObjectInterface>) receiver slot: (NSString *) member accel: (int) accel identifier: (int) identifier index: (int) index
{
	identifier = [self insertItem: text identifier: identifier index: index];
	[self connectItem: identifier receiver: receiver slot: member];
	[self setAccel: accel identifier: identifier];
	return identifier;
}

- (int) insertPixmapItem: (QPixmap *) pixmap receiver: (<QObjectInterface>) receiver slot: (NSString *) member accel: (int) accel
{
	int identifier = [self insertPixmapItem: pixmap identifier: -1 index: -1];
	[self connectItem: identifier receiver: receiver slot: member];
	[self setAccel: accel identifier: identifier];
	return identifier;
}


- (int) insertPixmapItem: (QPixmap *) pixmap text: (NSString *) text receiver: (<QObjectInterface>) receiver slot: (NSString *) member accel: (int) accel
{
	int identifier = [self insertPixmapItem: pixmap text: text identifier: -1 index: -1];
	[self connectItem: identifier receiver: receiver slot: member];
	[self setAccel: accel identifier: identifier];
	return identifier;
}

- (int) insertItem: (NSString *) text identifier: (int) identifier
{
	return [self insertItem: text identifier: identifier index: -1];
}

- (int) insertItem: (NSString *) text identifier: (int) identifier index: (int) index
{
	return (int) qt_QPopupMenu_insertItem3([self qt], (char *) [text cString], identifier, index);
}

- (int) insertPopupItem: (NSString *) text popup: (QPopupMenu *) popup
{
	return [self insertPopupItem: text popup: popup identifier: -1 index: -1];
}

- (int) insertPopupItem: (NSString *) text popup: (QPopupMenu *) popup identifier: (int) identifier index: (int) index
{
	return (int) qt_QPopupMenu_insertItem4([self qt], (char *) [text cString], [popup qt], identifier, index);
}

- (int) insertPixmapItem: (QPixmap *) pixmap identifier: (int) identifier index: (int) index
{
	return (int) qt_QPopupMenu_insertItem5([self qt], [pixmap qt], identifier, index);
}


- (int) insertPixmapItem: (QPixmap *) pixmap popup: (QPopupMenu *) popup identifier: (int) identifier index: (int) index
{
	return (int) qt_QPopupMenu_insertItem6([self qt], [pixmap qt], [popup qt], identifier, index);
}

- (int) insertPixmapItem:  (QPixmap *) pixmap text: (NSString *) text identifier: (int) identifier index: (int) index
{
	return (int) qt_QPopupMenu_insertItem7([self qt], [pixmap qt], (char *) [text cString], identifier, index);
}

- (int) insertPopupItem: (QPixmap *) pixmap text: (NSString *)text popup: (QPopupMenu *) popup identifier: (int) identifier index: (int) index
{
	return (int) qt_QPopupMenu_insertItem8([self qt], [pixmap qt], (char *) [text cString], [popup qt], identifier, index);
}


- insertSeparator
{
	qt_QPopupMenu_insertSeparator([self qt], -1);
	return self;
}

- insertSeparator: (int) index
{
	qt_QPopupMenu_insertSeparator([self qt], index);
	return self;
}


- removeItem: (int) identifier
{
	qt_QMenuData_removeItem(qt_QPopupMenu_QMenuData([self qt]), identifier);
	return self;
}

		
- removeItemAt: (int) index
{
	qt_QMenuData_removeItemAt(qt_QPopupMenu_QMenuData([self qt]), index);
	return self;
}


- clear
{
	qt_QMenuData_clear(qt_QPopupMenu_QMenuData([self qt]));
	return self;
}


- (int) accel: (int) identifier
{
	return (int) qt_QMenuData_accel(qt_QPopupMenu_QMenuData([self qt]), identifier);
}


- setAccel: (int) key identifier: (int) identifier
{
	qt_QMenuData_setAccel(qt_QPopupMenu_QMenuData([self qt]), key, identifier);
	return self;
}


- (NSString *) text: (int) identifier
{
	return (NSString *) [NSString stringWithCString: qt_QMenuData_text(qt_QPopupMenu_QMenuData([self qt]), identifier)];
}


- (QPixmap *) pixmap: (int) identifier
{
	return (QPixmap *) [QObject objectForQtKey: qt_QMenuData_pixmap(qt_QPopupMenu_QMenuData([self qt]), identifier)];
}


- changeItem: (NSString *) text identifier: (int) identifier
{
	qt_QMenuData_changeItem(qt_QPopupMenu_QMenuData([self qt]), (char *) [text cString], identifier);
	return self;
}


- changePixmapItem: (QPixmap *) pixmap identifier: (int) identifier
{
	qt_QMenuData_changeItem1(qt_QPopupMenu_QMenuData([self qt]), [pixmap qt], identifier);
	return self;
}


- changePixmapItem: (QPixmap *) pixmap text: (NSString *) text identifier: (int) identifier
{
	qt_QMenuData_changeItem2(qt_QPopupMenu_QMenuData([self qt]), [pixmap qt], (char *) [text cString], identifier);
	return self;
}


- (BOOL) isItemEnabled: (int) identifier
{
	return (BOOL) qt_QMenuData_isItemEnabled(qt_QPopupMenu_QMenuData([self qt]), identifier);
}


- setItemEnabled: (int) identifier enable: (BOOL) yn
{
	qt_QPopupMenu_setItemEnabled([self qt], identifier, yn);
	return self;
}


- (BOOL) isItemChecked: (int) identifier
{
	return (BOOL) qt_QPopupMenu_isItemChecked([self qt], identifier);
}


- setItemChecked: (int) identifier check: (BOOL) yn
{
	qt_QPopupMenu_setItemChecked([self qt], identifier, yn);
	return self;
}


- updateItem: (int) identifier
{
	qt_QPopupMenu_updateItem([self qt], identifier);
	return self;
}


- (int) indexOf: (int) identifier
{
	return (int) qt_QMenuData_indexOf(qt_QPopupMenu_QMenuData([self qt]), identifier);
}


- (int) idAt: (int) index
{
	return (int) qt_QMenuData_idAt(qt_QPopupMenu_QMenuData([self qt]), index);
}


- setId: (int) index identifier: (int) identifier
{
	qt_QMenuData_setId(qt_QPopupMenu_QMenuData([self qt]), index, identifier);
	return self;
}


- (BOOL) connectItem: (int) identifier receiver: (<QObjectInterface,QtSupport>) receiver slot: (NSString *) member
{
	qt_Slot *	slotId;
	
	if ([member hasSuffix: @")"]) {
		return (BOOL) qt_QPopupMenu_connectItem(	[self qt],
												identifier,
												[receiver qt],
												(char *)[[NSString stringWithFormat: @"1%@", member] cString] );
	} else {
		slotId = (qt_Slot *) [QObject slotIdForReceiver: receiver slot: member];
		
		return (BOOL) qt_QPopupMenu_connectItem(	[self qt],
												identifier,
												(qt_QObject *) slotId,
												([member hasSuffix: @":"]) ? "1UserDataSlot(int)" : "1UserDataSlot()" );
	}
}


- (BOOL) disconnectItem: (int) identifier receiver: (<QObjectInterface,QtSupport>) receiver slot: (NSString *) member
{
	return (BOOL) qt_QPopupMenu_disconnectItem([self qt], identifier, [receiver qt], (char *) [member cString]);
}


@end
