/***************************************************************************
                          QSpinBox.m  -  description
                             -------------------
    begin                : Sun Apr 16 2000
    copyright            : (C) 2000 by Richard Dale
    email                : Richard_Dale@tipitina.demon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtobjc/QSpinBox.h>
#include <qtobjc/QValidator.h>
#include <qtc/qtc.h>

@implementation QSpinBox

- initWithParent: (QWidget *) parent name: (NSString *) name
{
	[self setQt: qt_new_QSpinBox([parent qt], (char *) [name cString])];	
	return self;
}


- initWithMinValue: (int) minValue maxValue: (int) maxValue step: (int) step parent: (QWidget *) parent name: (NSString *) name
{
	[self setQt: qt_new_QSpinBox1(minValue, maxValue, step, [parent qt], (char *) [name cString])];
	[super init];	
	return self;
}


- (void) dealloc
{
	qt_del_QSpinBox([self qt]);	
	[super dealloc];	
	return;
}


- (NSString *) text
{
	return (NSString *) [NSString stringWithCString: qt_QSpinBox_text([self qt])];
}


- (NSString *) prefix
{
	return (NSString *) [NSString stringWithCString: qt_QSpinBox_prefix([self qt])];
}


- (NSString *) suffix
{
	return (NSString *) [NSString stringWithCString: qt_QSpinBox_suffix([self qt])];
}


- (NSString *) cleanText
{
	return (NSString *) [NSString stringWithCString: qt_QString_data(qt_QSpinBox_cleanText([self qt]))];
}


- setSpecialValueText: (NSString *) text
{
	qt_QSpinBox_setSpecialValueText([self qt], (char *) [text cString]);
	return self;
}


- (NSString *) specialValueText
{
	return (NSString *) [NSString stringWithCString: qt_QSpinBox_specialValueText([self qt])];
}


- setWrapping: (BOOL) on
{
	qt_QSpinBox_setWrapping([self qt], on);
	return self;
}


- (BOOL) wrapping
{
	return (BOOL) qt_QSpinBox_wrapping([self qt]);
}


- setValidator: (QValidator *) v
{
	qt_QSpinBox_setValidator([self qt], [v qt]);
	return self;
}


- (QSize *) sizeHint
{
	return (QSize *) [QObject objectForQtKey: qt_QSpinBox_sizeHint([self qt]) withClass: [QSize class]];
}


- setValue: (int) value
{
	qt_QSpinBox_setValue([self qt], value);
	return self;
}


- setPrefix: (NSString *) text
{
	qt_QSpinBox_setPrefix([self qt], (char *) [text cString]);
	return self;
}


- setSuffix: (NSString *) text
{
	qt_QSpinBox_setSuffix([self qt], (char *) [text cString]);
	return self;
}


- stepUp
{
	qt_QSpinBox_stepUp([self qt]);
	return self;
}


- stepDown
{
	qt_QSpinBox_stepDown([self qt]);
	return self;
}

@end

@implementation QSpinBox (QRangeControlImplementation)

- (int) value
{
	return (int) qt_QRangeControl_value(qt_QSpinBox_QRangeControl([self qt]));
}


- setValue: (int) value
{
	qt_QRangeControl_setValue(qt_QSpinBox_QRangeControl([self qt]), value);
	return self;
}


- addPage
{
	qt_QRangeControl_addPage(qt_QSpinBox_QRangeControl([self qt]));
	return self;
}


- subtractPage
{
	qt_QRangeControl_subtractPage(qt_QSpinBox_QRangeControl([self qt]));
	return self;
}


- addLine
{
	qt_QRangeControl_addLine(qt_QSpinBox_QRangeControl([self qt]));
	return self;
}


- subtractLine
{
	qt_QRangeControl_subtractLine(qt_QSpinBox_QRangeControl([self qt]));
	return self;
}


- (int) minValue
{
	return (int) qt_QRangeControl_minValue(qt_QSpinBox_QRangeControl([self qt]));
}


- (int) maxValue
{
	return (int) qt_QRangeControl_maxValue(qt_QSpinBox_QRangeControl([self qt]));
}


- setRangeMinValue: (int) minValue maxValue: (int) maxValue
{
	qt_QRangeControl_setRange(qt_QSpinBox_QRangeControl([self qt]), minValue, maxValue);
	return self;
}


- (int) lineStep
{
	return (int) qt_QRangeControl_lineStep(qt_QSpinBox_QRangeControl([self qt]));
}


- (int) pageStep
{
	return (int) qt_QRangeControl_pageStep(qt_QSpinBox_QRangeControl([self qt]));
}


- setLineStep: (int) line pageStep: (int) page
{
	qt_QRangeControl_setSteps(qt_QSpinBox_QRangeControl([self qt]), line, page);
	return self;
}

@end

