/***************************************************************************
                          QWidget.h  -  description
                             -------------------
    begin                : Fri Mar 3 2000
    copyright            : (C) 2000 by Richard Dale
    email                : Richard_Dale@tipitina.demon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QWIDGET_H
#define QWIDGET_H

#include <Foundation/NSString.h>

#include <qtobjc/qwindowdefs.h>
#include <qtobjc/QObject.h>
#include <qtobjc/QPaintDevice.h>
#include <qtobjc/QSize.h>
#include <qtobjc/QPalette.h>
#include <qtobjc/QFontMetrics.h>
#include <qtobjc/QFontInfo.h>
#include <qtobjc/QCursor.h>

//	BackgroundMode
#define FixedColor				0
#define FixedPixmap				1
#define NoBackground				2
#define PaletteForeground			3
#define PaletteBackground			4
#define PaletteLight				5
#define PaletteMidlight			6
#define PaletteDark				7
#define PaletteMid				8
#define PaletteText				9
#define PaletteBase				10

//	PropagationMode
#define NoChildren				0
#define AllChildren				1
#define SameFont					2
#define SamePalette				SameFont

//	FocusPolicy
#define NoFocus					0,
#define TabFocus					0x1
#define ClickFocus				0x2
#define StrongFocus				0x3

/**
  *@author Richard Dale
  */
@class QPixmap;
@class QBitmap;

@interface  QWidget : QPaintDevice

- init;
- initWithParent: (QWidget *) parent;
- initWithParent: (QWidget *) parent name: (NSString *) name flags: (WFlags) f;
- (void) dealloc;

- (WId) winId;
- (GUIStyle) style;
- setStyle: (GUIStyle) style;
- (BOOL) isTopLevel;
- (BOOL) isModal;
- (BOOL) isPopup;
- (BOOL) isDesktop;
- (BOOL) isEnabled;
- (BOOL) isEnabledTo: (QWidget*) widget;
- (BOOL) isEnabledToTLW;
- setEnabled: (BOOL) yn;
- (QRect *) frameGeometry;
- (QRect *) geometry;
- (int) x;
- (int) y;
- (QPoint *) pos;
- (QSize *) size;
- (int) width;
- (int) height;
- (QRect *) rect;
- (QRect *) childrenRect;
- (QSize *) minimumSize;
- (QSize *) maximumSize;
- setMinimumSize: (QSize *) size;
- setMinimumSize: (int) minw : (int) minh;
- setMaximumSize: (QSize *) size;
- setMaximumSize: (int) maxw : (int) maxh;
- setMinimumWidth: (int) minw;
- setMinimumHeight: (int) minh;
- setMaximumWidth: (int) maxw;
- setMaximumHeight: (int) maxh;
- setBitmapMask: (QBitmap *) bitmap;
- setRegionMask: (QRegion *) region;
- clearMask;
- (QSize *) sizeIncrement;
- setSizeIncrement: (QSize *) size;
- setSizeIncrement: (int) w : (int) h;
- setFixedSize: (QSize *) size;
- setFixedSize: (int) w : (int) h;
- setFixedWidth: (int) w;
- setFixedHeight: (int) h;
- (QPoint *) mapToGlobal: (QPoint *) point;
- (QPoint *) mapFromGlobal: (QPoint *) point;
- (QPoint *) mapToParent: (QPoint *) point;
- (QPoint *) mapFromParent: (QPoint *) point;
- (QWidget *) topLevelWidget;
- (int) backgroundMode;
- setBackgroundMode: (int) mode;
- (QColor *) backgroundColor;
- (QColor *) foregroundColor;
- setBackgroundColor: (QColor *) color;
- (QPixmap *) backgroundPixmap;
- setBackgroundPixmap: (QPixmap *) pixmap;
- (QColorGroup *) colorGroup;
- (QPalette *) palette;
- setPalette: (QPalette *) palette;
- (QFont *) font;
- setFont: (QFont *) font;
- (QFontMetrics *) fontMetrics;
- (QFontInfo *) fontInfo;
- (int) fontPropagation;
- setFontPropagation: (int) mode;
- (int) palettePropagation;
- setPalettePropagation: (int) mode;
- (QCursor *) cursor;
- setCursor: (QCursor *) cursor;
- (NSString *) caption;
- (QPixmap *) icon;
- (NSString *) iconText;
- (BOOL) hasMouseTracking;
- setCaption: (NSString *) caption;
- setIcon: (QPixmap *) pixmap;
- setIconText: (NSString *) text;
- setMouseTracking: (BOOL) enable;
- (BOOL) isActiveWindow;
- setActiveWindow;
- (BOOL) isFocusEnabled;
- (int) focusPolicy;
- setFocusPolicy: (int) policy;
- (BOOL) hasFocus;
- setFocus;
- clearFocus;
+ setTabOrderFirst: (QWidget *) first second: (QWidget *) second;
- setFocusProxy: (QWidget *) policy;
- (QWidget *) focusProxy;
- grabMouse;
- grabMouse: (QCursor *) cursor;
- releaseMouse;
- grabKeyboard;
- releaseKeyboard;
+ (QWidget *) mouseGrabber;
+ (QWidget *) keyboardGrabber;
- (BOOL) isUpdatesEnabled;
- setUpdatesEnabled: (BOOL) enable;
- update;
- update: (int) x : (int) y : (int) w : (int) h;
- update: (QRect *) rect;
- repaint;
- repaint: (BOOL) erase;
- repaint: (int) x : (int) y : (int) w : (int) h erase: (BOOL) erase;
- repaint: (QRect *) rect erase: (BOOL) erase;
- show;
- hide;
- iconify;
- (BOOL) close: (BOOL) forceKill;
- (BOOL) isVisible;
- (BOOL) isVisible: (QWidget *) widget;
- (BOOL) isVisibleToTLW;
- raise;
- lower;
- move: (int) x : (int) y;
- move: (QPoint *) point;
- resize: (int) w : (int) h;
- resize: (QSize *) size;
- setGeometry: (int) x : (int) y : (int) w : (int) h;
- setGeometry: (QRect *) rect;
- (QSize *) sizeHint;
- adjustSize;
- recreate: (QWidget *) parent flags: (WFlags) flags point: (QPoint *) point showIt: (BOOL) yn;
- erase;
- erase: (int) x : (int) y : (int) w : (int) h;
- erase: (QRect *) rect;
- scroll: (int) dx : (int) dy;
- drawText: (int) x : (int) y text: (NSString *) text;
- drawText: (QPoint *) point text: (NSString *) text;
- (QWidget *) focusWidget;
- setAcceptDrops: (BOOL) on;
- (BOOL) acceptDrops;
- (QWidget *) parentWidget;
- (BOOL) testWFlags: (WFlags) n;
+ (QWidget *) find: (uint) windowId;

@end

@interface QWidget (QObjectImplementation)  <QObjectInterface>
@end

#endif

