/***************************************************************************
                          QWidget.m  -  description
                             -------------------
    begin                : Wed Mar 8 2000
    copyright            : (C) 2000 by Richard Dale
    email                : Richard_Dale@tipitina.demon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <Foundation/NSAutoreleasePool.h>

#include <qtobjc/QWidget.h>
#include <qtobjc/QEvent.h>
#include <qtc/qtc.h>

static int
qtEventDelegate(void * qtWidget, char * methodName,void * qtEvent)
{
	QWidget *	widget;
	QEvent *		event;
	SEL			eventMethod;                                                                          	
	NSAutoreleasePool * eventPool;
		
	eventMethod = sel_get_uid([[NSString stringWithFormat: @"%s:", methodName] cString]);
	widget = [QObject objectForQtKey: qtWidget];
	
	if ([[widget class] instancesRespondToSelector: eventMethod]) {
/*		NSLog(	@"widget %@ responds to methodName: '%s' ('%s') type: %d",
				widget,
				methodName,
				sel_get_name(eventMethod),
				qt_QEvent_type(qtEvent) );
*/		
		eventPool = [[NSAutoreleasePool alloc] init];
		
		switch (qt_QEvent_type(qtEvent)) {
		case Event_Timer:
			event = [[[QTimerEvent alloc] init] autorelease];
			break;
		case Event_MouseButtonPress:
		case Event_MouseButtonRelease:
		case Event_MouseButtonDblClick:
		case Event_MouseMove:
			event = [[[QMouseEvent alloc] init] autorelease];
			break;
		case Event_KeyPress:
		case Event_KeyRelease:
			event = [[[QKeyEvent alloc] init] autorelease];
			break;
		case Event_FocusIn:
		case Event_FocusOut:
			event = [[[QFocusEvent alloc] init] autorelease];
			break;
		case Event_Enter:
		case Event_Leave:
			event = [[[QEvent alloc] init] autorelease];
			break;
		case Event_Paint:
			event = [[[QPaintEvent alloc] init] autorelease];
			break;
		case Event_Move:
			event = [[[QMoveEvent alloc] init] autorelease];
			break;
		case Event_Resize:
			event = [[[QResizeEvent alloc] init] autorelease];
			break;
		case Event_Close:
			event = [[[QCloseEvent alloc] init] autorelease];
			break;
		default:
			[eventPool release];
			return 0;
		}
		
		[event setQt: qtEvent];
		[widget performSelector: eventMethod withObject: event]; 	
		[eventPool release];
		return 1;
	}
	
//	NSLog(@"eventHandler() called: widget: '%@' '%s' selector: '%s'", widget, methodName, sel_get_name(eventMethod));
	return 0;
}

@implementation QWidget

+ (void) initialize
{
	qt_QWidget_registerEventDelegate(qtEventDelegate);
	return;
}

- init
{
	[super init];
	
	if ([self qt] == nil) {
		[self setQt: qt_new_QWidget(NULL, (char *) NULL, 0)];
	}
	
	[QObject setObject: self forQtKey: [self qt]];
	
	return self;
}

- initWithParent: (QWidget *) parent
{
	return [self initWithParent: parent name: [self description] flags: 0];
}

- initWithParent: (QWidget *) parent name: (NSString *) name flags: (WFlags) f
{
	[super init];
	[self setQt: qt_new_QWidget([parent qt], (char *) [name cString], f)];
	[QObject setObject: self forQtKey: [self qt]];
	return self;
}

- (void) dealloc
{
	[QObject removeQtKey: [self qt]];
	(void) qt_del_QWidget([self qt]);
	[super dealloc];
	return;
}

- (void *) paintDevice
{
	return qt_QWidget_QPaintDevice([self qt]);
}

- (WId) winId
{
	return (WId) qt_QWidget_winId([self qt]);
}


- (GUIStyle) style
{
	return (GUIStyle) qt_QWidget_style([self qt]);
}


- setStyle: (GUIStyle) style
{
	qt_QWidget_setStyle([self qt], style);
	return self;
}


- (BOOL) isTopLevel
{
	return (BOOL) qt_QWidget_isTopLevel([self qt]);
}


- (BOOL) isModal
{
	return (BOOL) qt_QWidget_isModal([self qt]);
}


- (BOOL) isPopup
{
	return (BOOL) qt_QWidget_isPopup([self qt]);
}


- (BOOL) isDesktop
{
	return (BOOL) qt_QWidget_isDesktop([self qt]);
}


- (BOOL) isEnabled
{
	return (BOOL) qt_QWidget_isEnabled([self qt]);
}


- (BOOL) isEnabledTo: (QWidget*) widget
{
//	return (BOOL) qt_QWidget_isEnabledTo([self qt], [widget qt]);
	return YES;
}


- (BOOL) isEnabledToTLW
{
//	return (BOOL) qt_QWidget_isEnabledToTLW([self qt]);
	return YES;
}


- setEnabled: (BOOL) yn
{
	qt_QWidget_setEnabled([self qt], yn);
	return self;
}


- (QRect *) frameGeometry
{
	return (QRect *) [QObject objectForQtKey: qt_QWidget_frameGeometry([self qt]) withClass: [QRect class]];
}


- (QRect *) geometry
{
	return (QRect *) [QObject objectForQtKey: qt_QWidget_geometry([self qt]) withClass: [QRect class]];
}


- (int) x
{
	return (int) qt_QWidget_x([self qt]);
}


- (int) y
{
	return (int) qt_QWidget_y([self qt]);
}


- (QPoint *) pos
{
	return (QPoint *) [QObject objectForQtKey: qt_QWidget_pos([self qt]) withClass: [QPoint class]];
}


- (QSize *) size
{
	return (QSize *) [QObject objectForQtKey: qt_QWidget_size([self qt]) withClass: [QSize class]];
}


- (int) width
{
	return (int) qt_QWidget_width([self qt]);
}


- (int) height
{
	return (int) qt_QWidget_height([self qt]);
}


- (QRect *) rect
{
	return (QRect *) [QObject objectForQtKey: qt_QWidget_rect([self qt]) withClass: [QRect class]];
}


- (QRect *) childrenRect
{
	return (QRect *) [QObject objectForQtKey: qt_QWidget_childrenRect([self qt]) withClass: [QRect class]];
}


- (QSize *) minimumSize
{
	return (QSize *) [QObject objectForQtKey: qt_QWidget_minimumSize([self qt]) withClass: [QSize class]];
}


- (QSize *) maximumSize
{
	return (QSize *) [QObject objectForQtKey: qt_QWidget_maximumSize([self qt]) withClass: [QSize class]];
}


- setMinimumSize: (QSize *) size
{
	qt_QWidget_setMinimumSize([self qt], [size qt]);
	return self;
}


- setMinimumSize: (int) minw : (int) minh
{
	qt_QWidget_setMinimumSize1([self qt], minw, minh);
	return self;
}


- setMaximumSize: (QSize *) size
{
	qt_QWidget_setMaximumSize([self qt], [size qt]);
	return self;
}


- setMaximumSize: (int) maxw : (int) maxh
{
	qt_QWidget_setMaximumSize1([self qt], maxw, maxh);
	return self;
}


- setMinimumWidth: (int) minw
{
	qt_QWidget_setMinimumWidth([self qt], minw);
	return self;
}


- setMinimumHeight: (int) minh
{
	qt_QWidget_setMinimumHeight([self qt], minh);
	return self;
}


- setMaximumWidth: (int) maxw
{
	qt_QWidget_setMaximumWidth([self qt], maxw);
	return self;
}


- setMaximumHeight: (int) maxh
{
	qt_QWidget_setMaximumHeight([self qt], maxh);
	return self;
}


- setBitmapMask: (QBitmap *) bitmap
{
//	qt_QWidget_setBitmapMask([self qt], [bitmap qt]);
	return self;
}


- setRegionMask: (QRegion *) region
{
//	qt_QWidget_setRegionMask([self qt], [region qt]);
	return self;
}


- clearMask
{
//	qt_QWidget_clearMask([self qt]);
	return self;
}


- (QSize *) sizeIncrement
{
	return (QSize*) [QObject objectForQtKey: qt_QWidget_sizeIncrement([self qt]) withClass: [QSize class]];
}


- setSizeIncrement: (QSize *) size
{
	qt_QWidget_setSizeIncrement([self qt], [size qt]);
	return self;
}


- setSizeIncrement: (int) w : (int) h
{
	qt_QWidget_setSizeIncrement1([self qt], w, h);
	return self;
}


- setFixedSize: (QSize *) size
{
	qt_QWidget_setFixedSize([self qt], [size qt]);
	return self;
}


- setFixedSize: (int) w : (int) h
{
	qt_QWidget_setFixedSize1([self qt], w, h);
	return self;
}


- setFixedWidth: (int) w
{
	qt_QWidget_setFixedWidth([self qt], w);
	return self;
}


- setFixedHeight: (int) h
{
	qt_QWidget_setFixedHeight([self qt], h);
	return self;
}


- (QPoint *) mapToGlobal: (QPoint *) point
{
	return (QPoint *) [QObject objectForQtKey: qt_QWidget_mapToGlobal([self qt], [point qt]) withClass: [QPoint class]];
}


- (QPoint *) mapFromGlobal: (QPoint *) point
{
	return (QPoint *) [QObject objectForQtKey: qt_QWidget_mapFromGlobal([self qt], [point qt]) withClass: [QPoint class]];
}


- (QPoint *) mapToParent: (QPoint *) point
{
	return (QPoint *) [QObject objectForQtKey: qt_QWidget_mapToParent([self qt], [point qt]) withClass: [QPoint class]];
}


- (QPoint *) mapFromParent: (QPoint *) point
{
	return (QPoint *) [QObject objectForQtKey: qt_QWidget_mapFromParent([self qt], [point qt]) withClass: [QPoint class]];
}


- (QWidget *) topLevelWidget
{
	return (QWidget *) [QObject objectForQtKey: qt_QWidget_topLevelWidget([self qt])];
}


- (int) backgroundMode
{
	return (int) qt_QWidget_backgroundMode([self qt]);
}


- setBackgroundMode: (int) mode
{
	qt_QWidget_setBackgroundMode([self qt], mode);
	return self;
}


- (QColor *) backgroundColor
{
	return (QColor *) [QObject objectForQtKey: qt_QWidget_backgroundColor([self qt]) withClass: [QColor class]];
}


- (QColor *) foregroundColor
{
	return (QColor *) [QObject objectForQtKey: qt_QWidget_foregroundColor([self qt]) withClass: [QColor class]];
}


- setBackgroundColor: (QColor *) color
{
	qt_QWidget_setBackgroundColor([self qt], [color qt]);
	return self;
}


- (QPixmap *) backgroundPixmap
{
	return (QPixmap *) [QObject objectForQtKey: qt_QWidget_backgroundPixmap([self qt]) withClass: [QPixmap class]];
}


- setBackgroundPixmap: (QPixmap *) pixmap
{
	qt_QWidget_setBackgroundPixmap([self qt], [(<QtSupport>) pixmap qt]);
	return self;
}


- (QColorGroup *) colorGroup
{
	return (QColorGroup *) [QObject objectForQtKey: qt_QWidget_colorGroup([self qt]) withClass: [QColorGroup class]];
}


- (QPalette *) palette
{
	return (QPalette *) [QObject objectForQtKey: qt_QWidget_palette([self qt]) withClass: [QPalette class]];
}


- setPalette: (QPalette *) palette
{
	qt_QWidget_setPalette([self qt], [palette qt]);
	return self;
}


- (QFont *) font
{
	return (QFont *) [QObject objectForQtKey: qt_QWidget_font([self qt]) withClass: [QFont class]];
}


- setFont: (QFont *) font
{
	qt_QWidget_setFont([self qt], [font qt]);
	return self;
}


- (QFontMetrics *) fontMetrics
{
	return (QFontMetrics *) [QObject objectForQtKey: qt_QWidget_fontMetrics([self qt]) withClass: [QFontMetrics class]];
}


- (QFontInfo *) fontInfo
{
	return (QFontInfo *) [QObject objectForQtKey: qt_QWidget_fontInfo([self qt]) withClass: [QFontInfo class]];
}


- (int) fontPropagation
{
//	return (int) qt_QWidget_fontPropagation([self qt]);
	return 0;
}


- setFontPropagation: (int) mode
{
//	qt_QWidget_setFontPropagation([self qt], mode);
	return self;
}


- (int) palettePropagation
{
//	return (int) qt_QWidget_palettePropagation([self qt]);
	return 0;
}


- setPalettePropagation: (int) mode
{
//	qt_QWidget_setPalettePropagation([self qt], mode);
	return self;
}


- (QCursor *) cursor
{
	return (QCursor *) [QObject objectForQtKey: qt_QWidget_cursor([self qt]) withClass: [QCursor class]];
}


- setCursor: (QCursor *) cursor
{
	qt_QWidget_setCursor([self qt], [cursor qt]);
	return self;
}


- (NSString *) caption
{
	return (NSString *) [NSString stringWithCString: qt_QWidget_caption([self qt])];
}


- (QPixmap *) icon
{
	return (QPixmap *) [QObject objectForQtKey: qt_QWidget_icon([self qt]) withClass: [QPixmap class]];
}


- (NSString *) iconText
{
	return (NSString *) [NSString stringWithCString: qt_QWidget_iconText([self qt])];
}


- (BOOL) hasMouseTracking
{
	return (BOOL) qt_QWidget_hasMouseTracking([self qt]);
}


- setCaption: (NSString *) caption
{
	qt_QWidget_setCaption([self qt], (char *) [caption cString]);
	return self;
}


- setIcon: (QPixmap *) pixmap
{
	qt_QWidget_setIcon([self qt], [(<QtSupport>) pixmap qt]);
	return self;
}


- setIconText: (NSString *) text
{
	qt_QWidget_setIconText([self qt], (char *) [text cString]);
	return self;
}


- setMouseTracking: (BOOL) enable
{
	qt_QWidget_setMouseTracking([self qt], enable);
	return self;
}


- (BOOL) isActiveWindow
{
	return (BOOL) qt_QWidget_isActiveWindow([self qt]);
}


- setActiveWindow
{
	qt_QWidget_setActiveWindow([self qt]);
	return self;
}


- (BOOL) isFocusEnabled
{
	return (BOOL) qt_QWidget_isFocusEnabled([self qt]);
}


- (int) focusPolicy
{
	return (int) qt_QWidget_focusPolicy([self qt]);
}


- setFocusPolicy: (int) policy
{
	qt_QWidget_setFocusPolicy([self qt], policy);
	return self;
}


- (BOOL) hasFocus
{
	return (BOOL) qt_QWidget_hasFocus([self qt]);
}


- setFocus
{
	qt_QWidget_setFocus([self qt]);
	return self;
}


- clearFocus
{
	qt_QWidget_clearFocus([self qt]);
	return self;
}


+ setTabOrderFirst: (QWidget *) first second: (QWidget *) second
{
	qt_QWidget_setTabOrder([first qt], [second qt]);
	return self;
}


- setFocusProxy: (QWidget *) proxy
{
	qt_QWidget_setFocusProxy([self qt], [proxy qt]);
	return self;
}


- (QWidget *) focusProxy
{
	return (QWidget *) [QObject objectForQtKey: qt_QWidget_focusProxy([self qt])];
}


- grabMouse
{
	qt_QWidget_grabMouse([self qt]);
	return self;
}


- grabMouse: (QCursor *) cursor
{
	qt_QWidget_grabMouse1([self qt], [cursor qt]);
	return self;
}


- releaseMouse
{
	qt_QWidget_releaseMouse([self qt]);
	return self;
}


- grabKeyboard
{
	qt_QWidget_grabKeyboard([self qt]);
	return self;
}


- releaseKeyboard
{
	qt_QWidget_releaseKeyboard([self qt]);
	return self;
}


+ (QWidget *) mouseGrabber
{
	return (QWidget *) [QObject objectForQtKey: qt_QWidget_mouseGrabber([self qt])];
}


+ (QWidget *) keyboardGrabber
{
	return (QWidget *) [QObject objectForQtKey: qt_QWidget_keyboardGrabber([self qt])];
}


- (BOOL) isUpdatesEnabled
{
	return (BOOL) qt_QWidget_isUpdatesEnabled([self qt]);
}


- setUpdatesEnabled: (BOOL) enable
{
	qt_QWidget_setUpdatesEnabled([self qt], enable);
	return self;
}


- update
{
	qt_QWidget_update([self qt]);
	return self;
}


- update: (int) x : (int) y : (int) w : (int) h
{
	qt_QWidget_update1([self qt], x, y, w, h);
	return self;
}


- update: (QRect *) rect
{
	qt_QWidget_update2([self qt], [rect qt]);
	return self;
}


- repaint
{
	[self repaint: YES];
	return self;
}

- repaint: (BOOL) erase
{
	qt_QWidget_repaint([self qt], erase);
	return self;
}

- repaint: (int) x : (int) y : (int) w : (int) h erase: (BOOL) erase
{
	qt_QWidget_repaint1([self qt], x, y, w, h, erase);
	return self;
}

- repaint: (QRect *) rect erase: (BOOL) erase
{
	qt_QWidget_repaint2([self qt], [rect qt], erase);
	return self;
}


- show
{
	qt_QWidget_show([self qt]);
	return self;
}


- hide
{
	qt_QWidget_hide([self qt]);
	return self;
}


- iconify
{
	qt_QWidget_iconify([self qt]);
	return self;
}


- (BOOL) close: (BOOL) forceKill
{
	return (BOOL) qt_QWidget_close([self qt], forceKill);
}


- (BOOL) isVisible
{
	return (BOOL) qt_QWidget_isVisible([self qt]);
}


- (BOOL) isVisible: (QWidget*) widget
{
//	return (BOOL) qt_QWidget_isVisible2([self qt], [widget qt]);
	return YES;
}


- (BOOL) isVisibleToTLW
{
	return (BOOL) qt_QWidget_isVisibleToTLW([self qt]);
}


- raise
{
	qt_QWidget_raise([self qt]);
	return self;
}


- lower
{
	qt_QWidget_lower([self qt]);
	return self;
}


- move: (int) x : (int) y
{
	qt_QWidget_move([self qt], x, y);
	return self;
}


- move: (QPoint *) point
{
	qt_QWidget_move1([self qt], [point qt]);
	return self;
}


- resize: (int) w : (int) h
{
	qt_QWidget_resize([self qt], w, h);
	return self;
}


- resize: (QSize *) size
{
	qt_QWidget_resize1([self qt], [size qt]);
	return self;
}


- setGeometry: (int) x : (int) y : (int) w : (int) h
{
	qt_QWidget_setGeometry([self qt], x, y, w, h);
	return self;
}


- setGeometry: (QRect *) rect
{
	qt_QWidget_setGeometry1([self qt], [rect qt]);
	return self;
}


- (QSize *) sizeHint
{
	return (QSize *) [QObject objectForQtKey: qt_QWidget_sizeHint([self qt])];
}


- adjustSize
{
	qt_QWidget_adjustSize([self qt]);
	return self;
}


- recreate: (QWidget *) parent flags: (WFlags) flags point: (QPoint *) point showIt: (BOOL) yn
{
	qt_QWidget_recreate([self qt], [parent qt], flags, [point qt], yn);
	return self;
}


- erase
{
	qt_QWidget_erase([self qt]);
	return self;
}


- erase: (int) x : (int) y : (int) w : (int) h
{
	qt_QWidget_erase1([self qt], x, y, w, h);
	return self;
}


- erase: (QRect *) rect
{
	qt_QWidget_erase2([self qt], [rect qt]);
	return self;
}


- scroll: (int) dx : (int) dy
{
	qt_QWidget_scroll([self qt], dx, dy);
	return self;
}


- drawText: (int) x : (int) y text: (NSString *) text
{
	qt_QWidget_drawText([self qt], x, y, (char *) [text cString]);
	return self;
}


- drawText: (QPoint *) point text: (NSString *) text
{
	qt_QWidget_drawText1([self qt], [point qt], (char *) [text cString]);
	return self;
}


- (QWidget *) focusWidget
{
	return (QWidget *) [QObject objectForQtKey: qt_QWidget_focusWidget([self qt])];
}


- setAcceptDrops: (BOOL) on
{
//	qt_QWidget_setAcceptDrops([self qt], on);
	return self;
}


- (BOOL) acceptDrops
{
//	return (BOOL) qt_QWidget_acceptDrops([self qt]);
	return YES;
}


- (QWidget *) parentWidget
{
	return (QWidget *) [QObject objectForQtKey: qt_QWidget_parentWidget([self qt])];
}


- (BOOL) testWFlags: (WFlags) n
{
	return (BOOL) qt_QWidget_testWFlags([self qt], n);
}


+ (QWidget *) find: (uint) windowId
{
	return (QWidget *) [QObject objectForQtKey: qt_QWidget_find(windowId)];
}

@end

@implementation QWidget (QObjectImplementation)

- initWithParent: (<QtSupport,QObjectInterface>) parent name: (NSString *) name
{
	return [self initWithParent: parent name: name flags: 0];
}
				
- (BOOL) event: (QEvent *) event
{
	return (BOOL) qt_QObject_event(	[self qt],
									(qt_QEvent *) [(<QtSupport>) event qt] );
}

- (BOOL) eventFilter: (QObject *) filter event: (QEvent *) event
{
	return (BOOL) qt_QObject_eventFilter(	[self qt],
										(qt_QObject *) [filter qt],
										(qt_QEvent *) [(<QtSupport>) event qt] );
}

- (NSString *) tr: (NSString *) string
{
//	return (NSString *) qt_QObject_tr([self qt], [string cString]);
	return nil;
}

- (NSString *) className
{
	return (NSString *) [NSString stringWithCString: qt_QObject_className([self qt])];
}

- (BOOL) isA: (NSString *) name
{
	return (BOOL) qt_QObject_isA([self qt], (char *) [name cString]);
}

- (BOOL) inherits: (NSString *) name
{
	return (BOOL) qt_QObject_inherits([self qt], (char *) [name cString]);
}

- setName: (NSString *) name
{
	qt_QObject_setName([self qt], (char *) [name cString]);
	return self;
}

- blockSignals: (BOOL) yn
{
	qt_QObject_blockSignals([self qt], yn);
	return self;
}

- (int) startTimer: (int) interval
{
	return (int) qt_QObject_startTimer([self qt], interval);
}

- killTimer: (int) identifier
{
	qt_QObject_killTimer([self qt], identifier);
	return self;
}

- killTimers
{
	qt_QObject_killTimers([self qt]);
	return self;
}

- (NSArray *) queryList: (NSString *) inheritsClass name: (NSString *) objname regexMatch: (BOOL) regex recursiveSearch: (BOOL) recursive
{
//	return (NSArray *) qt_QObject_queryList([self qt]);
	return nil;
}

- insertChild: (QObject *) child
{
	qt_QObject_insertChild([self qt], (qt_QObject *) [child qt]);
	return self;
}

- removeChild: (QObject *) child
{
	qt_QObject_removeChild([self qt], (qt_QObject *) [child qt]);
	return self;
}

- installEventFilter: (QObject *) filter
{
	qt_QObject_installEventFilter([self qt], (qt_QObject *) [filter qt]);
	return self;
}

- removeEventFilter: (QObject *) filter
{
	qt_QObject_removeEventFilter([self qt], (qt_QObject *) [filter qt]);
	return self;
}

+ (BOOL) connect: (QObject *) sender signal: (NSString *) signal receiver: (QObject *) receiver slot: (NSString *) member
{
	return [QObject connect: sender signal: signal receiver: receiver slot: member];
}

- (BOOL) connect: (QObject *) sender signal: (NSString *) signal slot: (NSString *) member
{
	return [QObject connect: sender signal: signal receiver: self slot: member];
}

+ (BOOL) disconnect: (QObject *) sender signal: (NSString *) signal receiver: (QObject *) receiver slot: (NSString *) member
{
//	return (BOOL) qt_QObject_disconnect([self qt]);
	return NO;
}

- (BOOL) disconnect: (NSString *) signal receiver: (QObject *) receiver slot: (NSString *) member
{
//	return (BOOL) qt_QObject_disconnect1([self qt]);
	return NO;
}

- (BOOL) disconnect: (QObject *) receiver slot: (NSString *) member
{
//	return (BOOL) qt_QObject_disconnect([self qt]);
	return NO;
}

- dumpObjectTree
{
	qt_QObject_dumpObjectTree([self qt]);
	return self;
}

- dumpObjectInfo
{
	qt_QObject_dumpObjectInfo([self qt]);
	return self;
}

- destroyed
{
//	qt_QObject_destroyed([self qt]);
	return self;
}

- (BOOL) activateFilters: (QEvent *) event
{
//	return (BOOL) qt_QObject_activateFilters([self qt]);
	return NO;
}

- (NSArray *) receivers: (NSString *) signal
{
//	return (NSArray *) qt_QObject_receivers([self qt]);
	return nil;
}

- activateSignal: (NSString *) signal
{
//	qt_QObject_activateSignal([self qt]);
	return self;
}

- activateSignal: (NSString *) signal shortValue: (short) s
{
//	qt_QObject_activateSignal([self qt]);
	return self;
}

- activateSignal: (NSString *) signal intValue: (int) i
{
//	qt_QObject_activateSignal([self qt]);
	return self;
}

- activateSignal: (NSString *) signal longValue: (long) l
{
//	qt_QObject_activateSignal([self qt]);
	return self;
}

- activateSignal: (NSString *) signal stringValue: (NSString *) str
{
//	qt_QObject_activateSignal([self qt]);
	return self;
}

- (QObject *) sender
{
//	return (QObject *) qt_QObject_sender([self qt]);
	return nil;
}

- initMetaObject
{
//	qt_QObject_initMetaObject([self qt]);
	return self;
}

- connectNotify: (NSString *) signal
{
//	qt_QObject_connectNotify([self qt]);
	return self;
}

- disconnectNotify: (NSString *) signal
{
//	qt_QObject_disconnectNotify([self qt]);
	return self;
}

- (BOOL) checkConnectArgs: (NSString *) signal receiver: (QObject *) receiver slot: (NSString *) member
{
//	return (BOOL) qt_QObject_checkConnectArgs([self qt]);
	return NO;
}

- emit: (NSString *) signal
{
	[QObject sender: self emit: signal];
	return self;
}

- emit: (NSString *) signal value: value
{
	[QObject sender: self emit: signal value: value];
	return self;
}

- emit: (NSString *) signal intValue: (int) value
{
	[QObject sender: self emit: signal intValue: value];
	return self;
}


@end

