/***************************************************************************
                          main.m  -  description
                             -------------------
    begin                : Fri Mar 17 15:04:11 EST 2000
    copyright            : (C) 2000 by Richard Dale
    email                : Richard_Dale@tipitina.demon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <Foundation/NSString.h>
#include <Foundation/NSAutoreleasePool.h>

#include <qtobjc/QApplication.h>
#include <qtobjc/QPainter.h>
#include <qtobjc/QPixmap.h>
#include <qtobjc/QWidget.h>
#include <qtobjc/QEvent.h>

/**
 * 'Programming with Qt' Example 2-4 page 30
 */

/**
 * A class that lets the user draw with the mouse. The
 * window knows how to redraw itself.
 */
@interface ScribbleWindow : QWidget
{
@private
	QPoint *		_last;
	QPixmap *	_buffer;
}

- init;
- (void) mousePressEvent: (QMouseEvent *) event;
- (void) mouseMoveEvent: (QMouseEvent *) event;
- (void) paintEvent: (QPaintEvent *) event;
- (void) resizeEvent: (QResizeEvent *) event;

@end

@implementation ScribbleWindow

/** The initializer. Rather simple for now. */
- init
{
	[super init];
	_buffer = [[QPixmap alloc] init];
	[self setBackgroundMode: NoBackground];
	return self;
}


/**
  * The method is called whenever the user presses the
  * mouse over the window. It just records the position of the mouse
  * at the time of the click.
  */
- (void) mousePressEvent: (QMouseEvent *) event
{
	_last = [[event pos] retain];
}


/**
  * This method is called whenever the user moves the mouse
  * while the mouse button is pressed. If we had called
  * - setMouseTracking: YES before, the method would also be called
  * when the mouse has moved without any button pressed. We know that
  * we haven't, and thus don't have to check whether any buttons are
  * pressed.
  */
- (void) mouseMoveEvent: (QMouseEvent *) event
{
	// create a QPainter object for drawing onto the window
	QPainter *	windowpainter = [[[QPainter alloc] init] autorelease];
	// create a QPainter object for drawing into an off-screen pixmap
	QPainter *	bufferpainter = [[[QPainter alloc] init] autorelease];
	
	// start painting
	[windowpainter begin: self]; 			// this painter paints onto the window
	[bufferpainter begin: _buffer];		// and this one paints in the buffer
	
	// draw a line in both the window and the buffer
	[windowpainter drawLineFrom: _last to: [event pos]];
	[bufferpainter drawLineFrom: _last to: [event pos]];
	
	// done with painting
	[windowpainter end];
	[bufferpainter end];
	
	// remember the current mouse position
	_last = [[event pos] retain];
}


/**
  * This method is called whenever the widget needs
  * painting, for example, when it has been obscured and then revealed again.
  */
- (void) paintEvent: (QPaintEvent *) event
{
	[QPaintDevice bitBlt: self dx: 0 dy: 0 source: _buffer];
}


/**
  * This method gets called whenever the window is resized. We
  * use it to make sure that the off-screen buffer is always the same
  * size as the window.
  * In order to retain the original scribbling, it is first copied
  * to a temporary buffer. After the main buffer has been resized and
  * filled with white, the image is copied from the temporary buffer to
  * the main buffer
  */
- (void) resizeEvent: (QResizeEvent *) event
{
	QPixmap * save = [[_buffer copy] autorelease];
	[_buffer resize: [event size]];
	[_buffer fill: [QColor white]];
	[QPaintDevice bitBlt: _buffer dx: 0 dy: 0 source: save];
}


@end


int main(int argc, char *argv[])
{
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	
	QApplication * myapp = [[QApplication alloc] initWithArgc: argc argv: argv];
	
	QWidget * mywidget = [[ScribbleWindow alloc] init];
	[mywidget setGeometry: 50 : 500 : 400 : 400];
	
	[myapp setMainWidget: mywidget];
	[mywidget show];
	
	return [myapp exec];
}
