/***************************************************************************
                          main.m  -  description
                             -------------------
    begin                : Fri Mar 17 15:04:11 EST 2000
    copyright            : (C) 2000 by Richard Dale
    email                : Richard_Dale@tipitina.demon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <Foundation/NSString.h>
#include <Foundation/NSAutoreleasePool.h>

#include <qtobjc/QApplication.h>
#include <qtobjc/QMenuBar.h>
#include <qtobjc/QMessageBox.h>
#include <qtobjc/QPainter.h>
#include <qtobjc/QPixmap.h>
#include <qtobjc/QPopupMenu.h>
#include <qtobjc/QWidget.h>

/**
 * 'Programming with Qt' Example 3-1 page 37
 */

 enum MenuIDs {
 	COLOR_MENU_ID_BLACK,
 	COLOR_MENU_ID_RED,
 	COLOR_MENU_ID_BLUE,
 	COLOR_MENU_ID_GREEN,
 	COLOR_MENU_ID_YELLOW };
 	

/**
 * A class that lets the user draw with the mouse. The
 * window knows how to redraw itself.
 */
@interface ScribbleWindow : QWidget
{
@private
	QPoint *		_last;
	QColor *		_currentcolor;
	
	QPixmap *	_buffer;
	QMenuBar *	_menubar;
	QPopupMenu *	_filemenu;
	QPopupMenu *	_colormenu;
	QPopupMenu *	_helpmenu;
}

- init;
- (void) dealloc;

- (void) mousePressEvent: (QMouseEvent *) event;
- (void) mouseMoveEvent: (QMouseEvent *) event;
- (void) paintEvent: (QPaintEvent *) event;
- (void) resizeEvent: (QResizeEvent *) event;

- (void) slotAbout;
- (void) slotAboutQt;
- (void) slotColorMenu: (int) item;

@end

@implementation ScribbleWindow

/** The initializer. Initializes the instance variables and the menu
 * system
 */
- init
{
	[super init];
	_buffer = [[QPixmap alloc] init];
	_currentcolor = [QColor black];
	
	// don't blank the window before repainting
	[self setBackgroundMode: NoBackground];
	
	/* The next lines build the menu bar. We first create the menus
	 * one by one, then add them to the menu bar. */
	_filemenu = [[QPopupMenu alloc] init];					// create a file menu
	[_filemenu insertItem: @"&Quit" receiver: [QApplication qApp] slot: @"quit()"];
	
	_colormenu = [[QPopupMenu alloc] init];  				// create a color menu
	[_colormenu insertItem: @"B&lack" identifier: COLOR_MENU_ID_BLACK];
	[_colormenu insertItem: @"&Red" identifier: COLOR_MENU_ID_RED];
	[_colormenu insertItem: @"&Blue" identifier: COLOR_MENU_ID_BLUE];
	[_colormenu insertItem: @"&Green" identifier: COLOR_MENU_ID_GREEN];
	[_colormenu insertItem: @"&Yellow" identifier: COLOR_MENU_ID_YELLOW];
	[	QObject	connect: _colormenu signal: @"activated(int)"
				receiver: self slot: @"slotColorMenu:" ];
	
	_helpmenu = [[QPopupMenu alloc] init];					// create a help menu
	[_helpmenu insertItem: @"&About QtScribble" receiver: self slot: @"slotAbout"];
	[_helpmenu insertItem: @"About &Qt" receiver: self slot: @"slotAboutQt"];
	
	_menubar = [[QMenuBar alloc] initWithParent: self];		// create a menu bar
	[_menubar insertPopupItem: @"&File" popup: _filemenu];
	[_menubar insertPopupItem: @"&Color" popup: _colormenu];
	[_menubar insertSeparator];
	[_menubar insertPopupItem: @"&Help" popup: _helpmenu];
	
	return self;
}

/**
  * The memory deallocation method. Does nothing for now.
  */
- (void) dealloc
{
	[super dealloc];
	return;
}

/**
  * The method is called whenever the user presses the
  * mouse over the window. It just records the position of the mouse
  * at the time of the click.
  */
- (void) mousePressEvent: (QMouseEvent *) event
{
	_last = [[event pos] retain];
}


/**
  * This method is called whenever the user moves the mouse
  * while the mouse button is pressed. If we had called
  * - setMouseTracking: YES before, the method would also be called
  * when the mouse has moved without any button pressed. We know that
  * we haven't, and thus don't have to check whether any buttons are
  * pressed.
  */
- (void) mouseMoveEvent: (QMouseEvent *) event
{
	// create a QPainter object for drawing onto the window
	QPainter *	windowpainter = [[[QPainter alloc] init] autorelease];
	// create a QPainter object for drawing into an off-screen pixmap
	QPainter *	bufferpainter = [[[QPainter alloc] init] autorelease];
	
	// start painting
	[windowpainter begin: self]; 			// this painter paints onto the window
	[bufferpainter begin: _buffer];		// and this one paints in the buffer
	
	// set a standard pen with the currently selected color
	[windowpainter setPenColor: _currentcolor];
	[bufferpainter setPenColor: _currentcolor];
	
	// draw a line in both the window and the buffer
	[windowpainter drawLineFrom: _last to: [event pos]];
	[bufferpainter drawLineFrom: _last to: [event pos]];
	
	// done with painting
	[windowpainter end];
	[bufferpainter end];
	
	// remember the current mouse position
	_last = [[event pos] retain];
}


/**
  * This method is called whenever the widget needs
  * painting, for example, when it has been obscured and then revealed again.
  */
- (void) paintEvent: (QPaintEvent *) event
{
	[QPaintDevice bitBlt: self dx: 0 dy: 0 source: _buffer];
}


/**
  * This method gets called whenever the window is resized. We
  * use it to make sure that the off-screen buffer is always the same
  * size as the window.
  * In order to retain the original scribbling, it is first copied
  * to a temporary buffer. After the main buffer has been resized and
  * filled with white, the image is copied from the temporary buffer to
  * the main buffer
  */
- (void) resizeEvent: (QResizeEvent *) event
{
	QPixmap * save = [[_buffer copy] autorelease];
	[_buffer resize: [event size]];
	[_buffer fill: [QColor white]];
	[QPaintDevice bitBlt: _buffer dx: 0 dy: 0 source: save];
}


- (void) slotAbout
{
	[	QMessageBox information: self caption: @"About QtScribble 2"
			text:	@"This is the QtScribble 2 application\n"
					@"Copyright 1998 by Matthias Kalle Dalheimer\n" ];
}

- (void) slotAboutQt
{
	[QMessageBox aboutQt: self caption: @"About Qt"];
}

- (void) slotColorMenu: (int) item
{
	switch (item) {
	case COLOR_MENU_ID_BLACK:
		_currentcolor = [QColor black];
		break;
	case COLOR_MENU_ID_RED:
		_currentcolor = [QColor red];
		break;
	case COLOR_MENU_ID_BLUE:
		_currentcolor = [QColor blue];
		break;
	case COLOR_MENU_ID_GREEN:
		_currentcolor = [QColor green];
		break;
	case COLOR_MENU_ID_YELLOW:
		_currentcolor = [QColor yellow];
		break;
	}
}


@end

int main(int argc, char *argv[])
{
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	
	QApplication * myapp = [[QApplication alloc] initWithArgc: argc argv: argv];
	
	QWidget * mywidget = [[ScribbleWindow alloc] init];
	[mywidget setGeometry: 50 : 500 : 400 : 400];
	
	[myapp setMainWidget: mywidget];
	[mywidget show];
	
	return [myapp exec];
}
